// $ANTLR 2.7.4: "java.g" -> "JavaRecognizer.java"$

/*
 * Copyright (c) 2000
 *      Jon Schewe.  All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * I'd appreciate comments/suggestions on the code jpschewe@mtu.net
 */
package net.mtu.eggplant.dbc;

import net.mtu.eggplant.util.StringPair;
import net.mtu.eggplant.util.Pair;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.LinkedList;
import java.util.LinkedHashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;

/**
 * Java parser for JonsAssert.  This file is generated by Antlr.
 *<pre>
 * Run 'java JonsAssert <directory full of java files>'
 *
 * Contributing authors:
 *      John Mitchell       johnm@non.net
 *      Terence Parr        parrt@magelang.com
 *      John Lilley         jlilley@empathy.com
 *      Scott Stanchfield   thetick@magelang.com
 *      Markus Mohnen       mohnen@informatik.rwth-aachen.de
 *      Peter Williams      pwilliams@netdynamics.com
 *
 * Version 1.00 December 9, 1997 -- initial release
 * Version 1.01 December 10, 1997
 *      fixed bug in octal def (0..7 not 0..8)
 * Version 1.10 August 1998 (parrt)
 *      added tree construction
 *      fixed definition of WS,comments for mac,pc,unix newlines
 *      added unary plus
 * Version 1.11 (Nov 20, 1998)
 *      Added "shutup" option to turn off last ambig warning.
 *      Fixed inner class def to allow named class defs as statements
 *      synchronized requires compound not simple statement
 *      add [] after builtInType DOT class in primaryExpression
 *      "const" is reserved but not valid..removed from modifiers
 *
 * Version 1.12 (Feb 2, 1999)
 *      Changed LITERAL_xxx to xxx in tree grammar.
 *      Updated java.g to use tokens {...} now for 2.6.0 (new feature).
 *
 * Version 1.13 (Apr 23, 1999)
 *      Didn't have (stat)? for else clause in tree parser.
 *      Didn't gen ASTs for interface extends.  Updated tree parser too.
 *      Updated to 2.6.0.
 * Version 1.14 (Jun 20, 1999)
 *      Allowed final/abstract on local classes.
 *      Removed local interfaces from methods
 *      Put instanceof precedence where it belongs...in relationalExpr
 *          It also had expr not type as arg; fixed it.
 *      Missing ! on SEMI in classBlock
 *      fixed: (expr) + "string" was parsed incorrectly (+ as unary plus).
 *      fixed: didn't like Object[].class in parser or tree parser
 *
 * This grammar is in the PUBLIC DOMAIN
 *
 * BUGS
 *</pre>
 *
 * <p>This parser has been modified from the original Recognizer to a pre-parser
 * that implements assertions in java as well as support JDK 1.4.</p>
 *
 * @version $Revision: 1.11 $
 */
public class JavaRecognizer extends antlr.LLkParser       implements JavaTokenTypes
 {


  private static final Log LOG = LogFactory.getLog(JavaRecognizer.class);

  public void print(final String s) {
    System.out.println("Parser: " + s);
  }

  /**
     set the symbol table object to use.
  **/
  public void setSymtab(final Symtab symtab) {
    _symtab = symtab;
  }

  private Symtab _symtab;

  /**
     Get the symbol table object being used.
  **/
  public Symtab getSymtab() {
    return _symtab;
  }
  
  /**
     Given the id Token from the implements clause find the package the
     interface is defined in.  Parse the interface for assertions and add
     an AssertInterface object to the current class so we can check against
     it later.
  **/
  protected void parseImplementedInterface(final Token t) {
    String interfaceName = t.getText();
    String packageName = "";
    if(interfaceName.indexOf('.') > 0) {
      //already qualified interface, break up into package and name
      int lastDot = interfaceName.lastIndexOf('.');
      packageName = interfaceName.substring(0, lastDot);
      interfaceName = interfaceName.substring(lastDot+1);
    }
    else {
      //Need to figure out the right package here, search through the
      //current package, _imports
    }

    //actually parse the file so we can check
    //against it later, if no file can be found then don't add it to the
    //class, otherwise add it to the current class with
    //getSymtab().getCurrentClass().addInterface(assertInterface)

  }
  
  /**
     add an assert.  This should get cached with the file object?
     @param asserts asserts, ordered as they appear in the code. 
  **/
  protected void addAsserts(final List asserts, final Token jdClose) {
    if(asserts != null && asserts.size() > 0) {
      int line = jdClose.getLine();
      int column = jdClose.getColumn()-1 + jdClose.getText().length();
      //System.out.println("adding assert"
      //  + " line: " + line
      //  + " column: " + column
      //  );
      StringBuffer codeFrag = new StringBuffer();
      Iterator iter = asserts.iterator();
      while(iter.hasNext()) {
        AssertToken assertToken = (AssertToken)iter.next();
        String code = CodeGenerator.generateAssertion(assertToken);
        codeFrag.append(code);
      }
      CodeFragment codeFragment = new CodeFragment(new CodePoint(line, column), codeFrag.toString(), CodeFragmentType.ASSERT);
      getSymtab().addCodeFragment(codeFragment);
    }
  }

  
  private List _invariants = new LinkedList();

  /**
     add invariants to this class

     @pre (invariant != null)
  **/
  protected void addInvariant(final Token invariant) {
    if(! (invariant instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + invariant.getClass());
    }
    _invariants.add(invariant);
  }

  /**
     get the invariants for this class.
     Invariants are in the order they appear in the source.
  **/
  protected List getInvariants() {
    return _invariants;
  }

  /**
     Clear the list of invariants for this class.
  **/
  protected void clearInvariants() {
    _invariants = new LinkedList();
  }


  private List _preConditions = new LinkedList();
  /**
     Get the list of preconditions that have been seen since the last clear.
     Preconditions are in the order they appear in the source.
  **/
  public List getPreConditions() {
    return _preConditions;
  }

  /**
     Add a precondition to the list of preconditions.
  **/
  public void addPreCondition(final Token pre) {
    if(! (pre instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + pre.getClass());
    }
    _preConditions.add(pre);
  }

  /**
     clear out the list of preconditions.
  **/
  public void clearPreConditions() {
    _preConditions = new LinkedList();
  }

  private List _postConditions = new LinkedList();
  /**
     Get the list of postconditions that have been seen since the last clear.
     Postconditions are in the order they appear in the source.
  **/
  public List getPostConditions() {
    return _postConditions;
  }

  /**
     clear out the list of postconditions.
  **/
  public void addPostCondition(final Token post) {
    if(! (post instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + post.getClass());
    }
    _postConditions.add(post);
  }

  /**
     clear out the list of postconditions.
  **/
  public void clearPostConditions() {
    _postConditions = new LinkedList();
  }    

protected JavaRecognizer(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public JavaRecognizer(TokenBuffer tokenBuf) {
  this(tokenBuf,2);
}

protected JavaRecognizer(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public JavaRecognizer(TokenStream lexer) {
  this(lexer,2);
}

public JavaRecognizer(ParserSharedInputState state) {
  super(state,2);
  tokenNames = _tokenNames;
}

	public final void unit() throws RecognitionException, TokenStreamException {
		
		
		pre();
		start();
		middle();
		end();
		match(Token.EOF_TYPE);
	}
	
	public final void pre() throws RecognitionException, TokenStreamException {
		
		
		uselessComments();
	}
	
	public final void start() throws RecognitionException, TokenStreamException {
		
		
		String packageName = null;
		
		
		{
		switch ( LA(1)) {
		case LITERAL_package:
		{
			packageName=packageDefinition();
			uselessComments();
			break;
		}
		case EOF:
		case FINAL:
		case ABSTRACT:
		case JAVADOC_OPEN:
		case SEMI:
		case LITERAL_import:
		case LITERAL_private:
		case LITERAL_public:
		case LITERAL_protected:
		case LITERAL_static:
		case LITERAL_transient:
		case LITERAL_native:
		case LITERAL_threadsafe:
		case LITERAL_synchronized:
		case LITERAL_volatile:
		case LITERAL_class:
		case LITERAL_interface:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			//stuff to do after finding packageName
			getSymtab().setCurrentPackageName(packageName);
			LOG.debug("just found the package: " + packageName);
			//Now we just need to check to make sure the destination file is older
			if(!getSymtab().getConfiguration().ignoreTimeStamp()) {
			if(!getSymtab().isDestinationOlderThanCurrentFile(packageName)) {
			// already parsed this file and force is off, skip it
			throw new FileAlreadyParsedException();
			}
			}
			
			//Put the print here so you don't see it unless we're really parsing the files
			if(getSymtab().getConfiguration().isVerbose()) {
			System.out.println("  " + getSymtab().getCurrentFile().getFile().getAbsolutePath());
			}
			
		}
	}
	
	public final void middle() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop7:
		do {
			if ((LA(1)==LITERAL_import)) {
				importDefinition();
				uselessComments();
			}
			else {
				break _loop7;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			
			clearInvariants();
			
		}
	}
	
	public final void end() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop11:
		do {
			if ((_tokenSet_0.member(LA(1)))) {
				{
				switch ( LA(1)) {
				case JAVADOC_OPEN:
				{
					invariantCondition();
					break;
				}
				case FINAL:
				case ABSTRACT:
				case SEMI:
				case LITERAL_private:
				case LITERAL_public:
				case LITERAL_protected:
				case LITERAL_static:
				case LITERAL_transient:
				case LITERAL_native:
				case LITERAL_threadsafe:
				case LITERAL_synchronized:
				case LITERAL_volatile:
				case LITERAL_class:
				case LITERAL_interface:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				typeDefinition();
			}
			else {
				break _loop11;
			}
			
		} while (true);
		}
	}
	
	public final void uselessComments() throws RecognitionException, TokenStreamException {
		
		
		boolean synPredMatched14 = false;
		if (((_tokenSet_1.member(LA(1))) && (_tokenSet_2.member(LA(2))))) {
			int _m14 = mark();
			synPredMatched14 = true;
			inputState.guessing++;
			try {
				{
				javadocComment();
				}
			}
			catch (RecognitionException pe) {
				synPredMatched14 = false;
			}
			rewind(_m14);
			inputState.guessing--;
		}
		if ( synPredMatched14 ) {
			{
			boolean synPredMatched17 = false;
			if (((_tokenSet_1.member(LA(1))) && (_tokenSet_2.member(LA(2))))) {
				int _m17 = mark();
				synPredMatched17 = true;
				inputState.guessing++;
				try {
					{
					invariantCondition();
					typeDefinition();
					}
				}
				catch (RecognitionException pe) {
					synPredMatched17 = false;
				}
				rewind(_m17);
				inputState.guessing--;
			}
			if ( synPredMatched17 ) {
			}
			else if ((LA(1)==JAVADOC_OPEN) && ((LA(2) >= JAVADOC_CLOSE && LA(2) <= INVARIANT_CONDITION))) {
				javadocComment();
				uselessComments();
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
		}
		else if ((_tokenSet_1.member(LA(1))) && (_tokenSet_2.member(LA(2)))) {
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
	}
	
/**
   Package statement: "package" followed by an identifier.
**/
	public final String  packageDefinition() throws RecognitionException, TokenStreamException {
		String packageName;
		
		
		Token id = null;
		packageName = null;
		
		
		match(LITERAL_package);
		id=identifier();
		match(SEMI);
		if ( inputState.guessing==0 ) {
			
			packageName = id.getText();
			
		}
		return packageName;
	}
	
	public final void importDefinition() throws RecognitionException, TokenStreamException {
		
		
		match(LITERAL_import);
		identifierStar();
		match(SEMI);
	}
	
/**
   This is a javadoc comment that we're looking for invariants in.
**/
	public final void invariantCondition() throws RecognitionException, TokenStreamException {
		
		Token  iv = null;
		
		match(JAVADOC_OPEN);
		{
		_loop23:
		do {
			switch ( LA(1)) {
			case INVARIANT_CONDITION:
			{
				iv = LT(1);
				match(INVARIANT_CONDITION);
				if ( inputState.guessing==0 ) {
					addInvariant(iv);
				}
				break;
			}
			case PRE_CONDITION:
			{
				match(PRE_CONDITION);
				break;
			}
			case POST_CONDITION:
			{
				match(POST_CONDITION);
				break;
			}
			case ASSERT_CONDITION:
			{
				match(ASSERT_CONDITION);
				break;
			}
			default:
			{
				break _loop23;
			}
			}
		} while (true);
		}
		match(JAVADOC_CLOSE);
	}
	
	public final void typeDefinition() throws RecognitionException, TokenStreamException {
		
		
		Set dummyMods;
		
		
		switch ( LA(1)) {
		case FINAL:
		case ABSTRACT:
		case LITERAL_private:
		case LITERAL_public:
		case LITERAL_protected:
		case LITERAL_static:
		case LITERAL_transient:
		case LITERAL_native:
		case LITERAL_threadsafe:
		case LITERAL_synchronized:
		case LITERAL_volatile:
		case LITERAL_class:
		case LITERAL_interface:
		{
			dummyMods=modifiers();
			{
			switch ( LA(1)) {
			case LITERAL_class:
			{
				classDefinition();
				break;
			}
			case LITERAL_interface:
			{
				interfaceDefinition();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case SEMI:
		{
			match(SEMI);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
/**
   This is a javadoc comment that we're not looking for any conditions in.
**/
	public final void javadocComment() throws RecognitionException, TokenStreamException {
		
		
		match(JAVADOC_OPEN);
		{
		_loop20:
		do {
			switch ( LA(1)) {
			case INVARIANT_CONDITION:
			{
				match(INVARIANT_CONDITION);
				break;
			}
			case PRE_CONDITION:
			{
				match(PRE_CONDITION);
				break;
			}
			case POST_CONDITION:
			{
				match(POST_CONDITION);
				break;
			}
			case ASSERT_CONDITION:
			{
				match(ASSERT_CONDITION);
				break;
			}
			default:
			{
				break _loop20;
			}
			}
		} while (true);
		}
		match(JAVADOC_CLOSE);
	}
	
/**
 A (possibly-qualified) java identifier.  We start with the first IDENT
   and expand its name by adding dots and following IDENTS

   @return a Token that represents this identifier, the text of the Token is
   equal to the whole identifier, including possible dots for a fully
   qualified java class name.
**/
	public final Token  identifier() throws RecognitionException, TokenStreamException {
		Token t;
		
		Token  id = null;
		Token  id2 = null;
		t = null;
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			t = id;
		}
		{
		_loop43:
		do {
			if ((LA(1)==DOT)) {
				match(DOT);
				id2 = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					t.setText(t.getText() + "." + id2.getText());
				}
			}
			else {
				break _loop43;
			}
			
		} while (true);
		}
		return t;
	}
	
/**
   Used for import statements.  Adds this package identifier to the list of
   imports in the symbol table.
**/
	public final void identifierStar() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		Token  id2 = null;
		
		String className = "";
		String packageName = "";
		
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			className = id.getText();
		}
		{
		_loop46:
		do {
			if ((LA(1)==DOT) && (LA(2)==IDENT)) {
				match(DOT);
				id2 = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					packageName += "." + className; className = id2.getText();
				}
			}
			else {
				break _loop46;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case DOT:
		{
			match(DOT);
			match(STAR);
			if ( inputState.guessing==0 ) {
				packageName += "." + className; className = null;
			}
			break;
		}
		case SEMI:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			
			// tell the symbol table about the import
			getSymtab().addImport(className, packageName);
			
		}
	}
	
/**
   @return the list of modifiers as Strings
**/
	public final Set  modifiers() throws RecognitionException, TokenStreamException {
		Set mods;
		
		
		mods = new HashSet(10);
		Token mod = null;
		
		
		{
		_loop31:
		do {
			if ((_tokenSet_3.member(LA(1)))) {
				mod=modifier();
				if ( inputState.guessing==0 ) {
					mods.add(mod.getText());
				}
			}
			else {
				break _loop31;
			}
			
		} while (true);
		}
		return mods;
	}
	
	public final void classDefinition() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		String name = null;
		Token superclass = null;
		
		
		match(LITERAL_class);
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
			if(getSymtab().getCurrentClass() != null) {
			name = getSymtab().getCurrentClass().getName() + "$" + id.getText();
			}
			else {
			name = id.getText();
			}
			
		}
		superclass=superClassClause();
		implementsClause();
		classBlock(name, false, false, superclass);
	}
	
	public final void interfaceDefinition() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		String name = null;
		
		
		match(LITERAL_interface);
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
			if(getSymtab().getCurrentClass() != null) {
			name = getSymtab().getCurrentClass().getName() + "$" + id.getText();
			}
			else {
			name = id.getText();
			}
			
		}
		interfaceExtends();
		classBlock(name, true, false, null);
	}
	
/** A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
	public final void declaration() throws RecognitionException, TokenStreamException {
		
		
		Set dummyMods;
		Token dummyType;
		
		
		dummyMods=modifiers();
		dummyType=typeSpec();
		variableDefinitions();
	}
	
/**
   @return the Token that represents this type spec
**/
	public final Token  typeSpec() throws RecognitionException, TokenStreamException {
		Token t;
		
		
		t = null;
		
		
		switch ( LA(1)) {
		case IDENT:
		{
			t=classTypeSpec();
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			t=builtInTypeSpec();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return t;
	}
	
	public final void variableDefinitions() throws RecognitionException, TokenStreamException {
		
		
		variableDeclarator();
		{
		_loop78:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				variableDeclarator();
			}
			else {
				break _loop78;
			}
			
		} while (true);
		}
	}
	
	public final Token  modifier() throws RecognitionException, TokenStreamException {
		Token t;
		
		Token  tprivate = null;
		Token  tpublic = null;
		Token  tprotected = null;
		Token  tstatic = null;
		Token  ttransient = null;
		Token  tfinal = null;
		Token  tabstract = null;
		Token  tnative = null;
		Token  tthreadsafe = null;
		Token  tsynchronized = null;
		Token  tvolatile = null;
		
		t = null;
		
		
		switch ( LA(1)) {
		case LITERAL_private:
		{
			tprivate = LT(1);
			match(LITERAL_private);
			if ( inputState.guessing==0 ) {
				t = tprivate;
			}
			break;
		}
		case LITERAL_public:
		{
			tpublic = LT(1);
			match(LITERAL_public);
			if ( inputState.guessing==0 ) {
				t = tpublic;
			}
			break;
		}
		case LITERAL_protected:
		{
			tprotected = LT(1);
			match(LITERAL_protected);
			if ( inputState.guessing==0 ) {
				t = tprotected;
			}
			break;
		}
		case LITERAL_static:
		{
			tstatic = LT(1);
			match(LITERAL_static);
			if ( inputState.guessing==0 ) {
				t = tstatic;
			}
			break;
		}
		case LITERAL_transient:
		{
			ttransient = LT(1);
			match(LITERAL_transient);
			if ( inputState.guessing==0 ) {
				t = ttransient;
			}
			break;
		}
		case FINAL:
		{
			tfinal = LT(1);
			match(FINAL);
			if ( inputState.guessing==0 ) {
				t = tfinal;
			}
			break;
		}
		case ABSTRACT:
		{
			tabstract = LT(1);
			match(ABSTRACT);
			if ( inputState.guessing==0 ) {
				t = tabstract;
			}
			break;
		}
		case LITERAL_native:
		{
			tnative = LT(1);
			match(LITERAL_native);
			if ( inputState.guessing==0 ) {
				t = tnative;
			}
			break;
		}
		case LITERAL_threadsafe:
		{
			tthreadsafe = LT(1);
			match(LITERAL_threadsafe);
			if ( inputState.guessing==0 ) {
				t = tthreadsafe;
			}
			break;
		}
		case LITERAL_synchronized:
		{
			tsynchronized = LT(1);
			match(LITERAL_synchronized);
			if ( inputState.guessing==0 ) {
				t = tsynchronized;
			}
			break;
		}
		case LITERAL_volatile:
		{
			tvolatile = LT(1);
			match(LITERAL_volatile);
			if ( inputState.guessing==0 ) {
				t = tvolatile;
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return t;
	}
	
/**
   @return the Token that represents this class type spec
**/
	public final Token  classTypeSpec() throws RecognitionException, TokenStreamException {
		Token id;
		
		
		id = null;
		
		
		id=identifier();
		{
		_loop35:
		do {
			if ((LA(1)==LBRACK)) {
				match(LBRACK);
				match(RBRACK);
				if ( inputState.guessing==0 ) {
					id.setText(id.getText() + "[]");
				}
			}
			else {
				break _loop35;
			}
			
		} while (true);
		}
		return id;
	}
	
/**
   @return the Token that represents this builtin type spec
**/
	public final Token  builtInTypeSpec() throws RecognitionException, TokenStreamException {
		Token id;
		
		
		id = null;
		
		
		id=builtInType();
		{
		_loop38:
		do {
			if ((LA(1)==LBRACK)) {
				match(LBRACK);
				match(RBRACK);
				if ( inputState.guessing==0 ) {
					id.setText(id.getText() + "[]");
				}
			}
			else {
				break _loop38;
			}
			
		} while (true);
		}
		return id;
	}
	
/**
   @return the Token that represents this builtin type
**/
	public final Token  builtInType() throws RecognitionException, TokenStreamException {
		Token t;
		
		Token  tvoid = null;
		Token  tboolean = null;
		Token  tbyte = null;
		Token  tchar = null;
		Token  tshort = null;
		Token  tint = null;
		Token  tfloat = null;
		Token  tlong = null;
		Token  tdouble = null;
		t = null;
		
		switch ( LA(1)) {
		case LITERAL_void:
		{
			tvoid = LT(1);
			match(LITERAL_void);
			if ( inputState.guessing==0 ) {
				t = tvoid;
			}
			break;
		}
		case LITERAL_boolean:
		{
			tboolean = LT(1);
			match(LITERAL_boolean);
			if ( inputState.guessing==0 ) {
				t = tboolean;
			}
			break;
		}
		case LITERAL_byte:
		{
			tbyte = LT(1);
			match(LITERAL_byte);
			if ( inputState.guessing==0 ) {
				t = tbyte;
			}
			break;
		}
		case LITERAL_char:
		{
			tchar = LT(1);
			match(LITERAL_char);
			if ( inputState.guessing==0 ) {
				t = tchar;
			}
			break;
		}
		case LITERAL_short:
		{
			tshort = LT(1);
			match(LITERAL_short);
			if ( inputState.guessing==0 ) {
				t = tshort;
			}
			break;
		}
		case LITERAL_int:
		{
			tint = LT(1);
			match(LITERAL_int);
			if ( inputState.guessing==0 ) {
				t = tint;
			}
			break;
		}
		case LITERAL_float:
		{
			tfloat = LT(1);
			match(LITERAL_float);
			if ( inputState.guessing==0 ) {
				t = tfloat;
			}
			break;
		}
		case LITERAL_long:
		{
			tlong = LT(1);
			match(LITERAL_long);
			if ( inputState.guessing==0 ) {
				t = tlong;
			}
			break;
		}
		case LITERAL_double:
		{
			tdouble = LT(1);
			match(LITERAL_double);
			if ( inputState.guessing==0 ) {
				t = tdouble;
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return t;
	}
	
/**
   @return the Token that represents this type
**/
	public final Token  type() throws RecognitionException, TokenStreamException {
		Token t;
		
		
		t = null;
		
		
		switch ( LA(1)) {
		case IDENT:
		{
			t=identifier();
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			t=builtInType();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return t;
	}
	
	public final Token  superClassClause() throws RecognitionException, TokenStreamException {
		Token id;
		
		id = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_extends:
		{
			match(LITERAL_extends);
			id=identifier();
			break;
		}
		case LCURLY:
		case LITERAL_implements:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return id;
	}
	
	public final void implementsClause() throws RecognitionException, TokenStreamException {
		
		Token id, id2;
		
		{
		switch ( LA(1)) {
		case LITERAL_implements:
		{
			match(LITERAL_implements);
			id=identifier();
			if ( inputState.guessing==0 ) {
				parseImplementedInterface(id);
			}
			{
			_loop69:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					id2=identifier();
					if ( inputState.guessing==0 ) {
						parseImplementedInterface(id2);
					}
				}
				else {
					break _loop69;
				}
				
			} while (true);
			}
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
/**
   This is the body of a class or interface.  You can have fields and extra semicolons,
   That's about it (until you see what a field is...)
**/
	public final void classBlock(
		 String name, boolean isInterface, boolean isAnonymous, Token superclass 
	) throws RecognitionException, TokenStreamException {
		
		Token  lc = null;
		Token  rc = null;
		
		lc = LT(1);
		match(LCURLY);
		if ( inputState.guessing==0 ) {
			
			getSymtab().startClass(name, getInvariants(), isInterface, isAnonymous, (superclass == null ? null : superclass.getText()));
			clearInvariants();
			
		}
		{
		_loop55:
		do {
			switch ( LA(1)) {
			case FINAL:
			case ABSTRACT:
			case JAVADOC_OPEN:
			case LITERAL_void:
			case LITERAL_boolean:
			case LITERAL_byte:
			case LITERAL_char:
			case LITERAL_short:
			case LITERAL_int:
			case LITERAL_float:
			case LITERAL_long:
			case LITERAL_double:
			case IDENT:
			case LITERAL_private:
			case LITERAL_public:
			case LITERAL_protected:
			case LITERAL_static:
			case LITERAL_transient:
			case LITERAL_native:
			case LITERAL_threadsafe:
			case LITERAL_synchronized:
			case LITERAL_volatile:
			case LITERAL_class:
			case LITERAL_interface:
			case LCURLY:
			{
				prePostField();
				break;
			}
			case SEMI:
			{
				match(SEMI);
				break;
			}
			default:
			{
				break _loop55;
			}
			}
		} while (true);
		}
		rc = LT(1);
		match(RCURLY);
		if ( inputState.guessing==0 ) {
			
			clearInvariants();
			getSymtab().finishClass(new CodePoint(rc.getLine(), rc.getColumn()-1));
			
		}
	}
	
	public final void interfaceExtends() throws RecognitionException, TokenStreamException {
		
		Token id, id2;
		
		{
		switch ( LA(1)) {
		case LITERAL_extends:
		{
			match(LITERAL_extends);
			id=identifier();
			{
			_loop65:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					id2=identifier();
				}
				else {
					break _loop65;
				}
				
			} while (true);
			}
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void prePostField() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop61:
		do {
			if ((LA(1)==JAVADOC_OPEN)) {
				match(JAVADOC_OPEN);
				prePosts();
				match(JAVADOC_CLOSE);
			}
			else {
				break _loop61;
			}
			
		} while (true);
		}
		field();
	}
	
	public final void prePosts() throws RecognitionException, TokenStreamException {
		
		Token  post = null;
		Token  pre = null;
		
		{
		_loop58:
		do {
			switch ( LA(1)) {
			case POST_CONDITION:
			{
				post = LT(1);
				match(POST_CONDITION);
				if ( inputState.guessing==0 ) {
					addPostCondition(post);
				}
				break;
			}
			case PRE_CONDITION:
			{
				pre = LT(1);
				match(PRE_CONDITION);
				if ( inputState.guessing==0 ) {
					addPreCondition(pre);
				}
				break;
			}
			case ASSERT_CONDITION:
			{
				match(ASSERT_CONDITION);
				break;
			}
			case INVARIANT_CONDITION:
			{
				match(INVARIANT_CONDITION);
				break;
			}
			default:
			{
				break _loop58;
			}
			}
		} while (true);
		}
	}
	
/**
 Now the various things that can be defined inside a class or interface...
 Note that not all of these are really valid in an interface (constructors,
   for example), and if this grammar were used for a compiler there would
   need to be some semantic checks to make sure we're doing the right thing...
**/
	public final void field() throws RecognitionException, TokenStreamException {
		
		Token  methodName = null;
		Token  semi = null;
		
		Set mods = null;
		Token retType = null;
		List params = null;
		boolean methodOrConstructor = false;
		CodePointPair startEnd = null;
		CodePointPair dummyStartEnd;
		Set thrownExceptions = null;
		Pair p = null;
		
		
		{
		if ((_tokenSet_4.member(LA(1))) && (_tokenSet_5.member(LA(2)))) {
			mods=modifiers();
			{
			switch ( LA(1)) {
			case LITERAL_class:
			{
				classDefinition();
				break;
			}
			case LITERAL_interface:
			{
				interfaceDefinition();
				break;
			}
			default:
				if ((LA(1)==IDENT) && (LA(2)==LPAREN)) {
					p=ctorHead();
					if ( inputState.guessing==0 ) {
						
						params = (List)p.getOne();
						thrownExceptions = (Set)p.getTwo();
						// needs to be before compoundStatement so that I can have it set for the addExit calls
						getSymtab().startMethod(null, getPreConditions(), getPostConditions(), params, null, mods);
						clearPreConditions();
						clearPostConditions();
						//print("just called startMethod for constructor");
						
					}
					startEnd=compoundStatement();
					if ( inputState.guessing==0 ) {
						
						getSymtab().finishMethod(startEnd, thrownExceptions);
						//print("Found finish: " + methodName);
						
					}
				}
				else if (((LA(1) >= LITERAL_void && LA(1) <= IDENT)) && (_tokenSet_6.member(LA(2)))) {
					retType=typeSpec();
					{
					if ((LA(1)==IDENT) && (LA(2)==LPAREN)) {
						methodName = LT(1);
						match(IDENT);
						match(LPAREN);
						params=parameterDeclarationList();
						if ( inputState.guessing==0 ) {
							
							// needs to be before compoundStatement so that I can have it set for the addExit calls
							getSymtab().startMethod(methodName.getText(), getPreConditions(), getPostConditions(), params, retType.getText(), mods);
							//print("just called startMethod: " + methodName.getText());
							clearPreConditions();
							clearPostConditions();
							
						}
						match(RPAREN);
						returnTypeBrackersOnEndOfMethodHead();
						{
						switch ( LA(1)) {
						case LITERAL_throws:
						{
							thrownExceptions=throwsClause();
							break;
						}
						case SEMI:
						case LCURLY:
						{
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						{
						switch ( LA(1)) {
						case LCURLY:
						{
							startEnd=compoundStatement();
							break;
						}
						case SEMI:
						{
							semi = LT(1);
							match(SEMI);
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						if ( inputState.guessing==0 ) {
							
							if(startEnd != null) {
							getSymtab().finishMethod(startEnd, thrownExceptions);
							}
							else {
							//abstract, native or interface method
							CodePoint close = new CodePoint(semi.getLine(), semi.getColumn()-1);
							getSymtab().finishMethod(new CodePointPair(close, close), null);
							}
							//print("Found finish: " + methodName);
							
						}
					}
					else if ((LA(1)==IDENT) && (_tokenSet_7.member(LA(2)))) {
						variableDefinitions();
						match(SEMI);
					}
					else {
						throw new NoViableAltException(LT(1), getFilename());
					}
					
					}
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		else if ((LA(1)==LITERAL_static) && (LA(2)==LCURLY)) {
			match(LITERAL_static);
			dummyStartEnd=compoundStatement();
		}
		else if ((LA(1)==LCURLY)) {
			dummyStartEnd=compoundStatement();
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
	}
	
/**
 <p>This is the header of a method.  It includes the name and parameters for
   the method.  This also watches for a list of exception classes in a
   "throws" clause.</p>

   <p>this is only used for constructors so I'm
   just going to return the params, the method name is known</p>

   @return the parameters as StringPair(type, name)
**/
	public final Pair  ctorHead() throws RecognitionException, TokenStreamException {
		Pair p;
		
		
		List params = null;
		Set thrownExceptions = null;
		p = null;
		
		
		{
		match(IDENT);
		match(LPAREN);
		params=parameterDeclarationList();
		match(RPAREN);
		{
		switch ( LA(1)) {
		case LITERAL_throws:
		{
			thrownExceptions=throwsClause();
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		}
		if ( inputState.guessing==0 ) {
			
			p = new Pair(params, thrownExceptions);
			
		}
		return p;
	}
	
/**
   @return a CodePointPair that represent the open and close curly braces
**/
	public final CodePointPair  compoundStatement() throws RecognitionException, TokenStreamException {
		CodePointPair startEnd;
		
		Token  lc = null;
		Token  rc = null;
		
		startEnd = null;
		
		
		{
		lc = LT(1);
		match(LCURLY);
		{
		_loop117:
		do {
			if ((_tokenSet_8.member(LA(1)))) {
				statement();
			}
			else {
				break _loop117;
			}
			
		} while (true);
		}
		rc = LT(1);
		match(RCURLY);
		}
		if ( inputState.guessing==0 ) {
			
			CodePoint start = new CodePoint(lc.getLine(), lc.getColumn()-1);
			CodePoint end = new CodePoint(rc.getLine(), rc.getColumn()-1);
			startEnd = new CodePointPair(start, end);
			
		}
		return startEnd;
	}
	
	public final List  parameterDeclarationList() throws RecognitionException, TokenStreamException {
		List params;
		
		
		params = new LinkedList();
		StringPair pd = null;
		
		
		{
		switch ( LA(1)) {
		case FINAL:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		{
			pd=parameterDeclaration();
			if ( inputState.guessing==0 ) {
				params.add(pd);
			}
			{
			_loop103:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					pd=parameterDeclaration();
					if ( inputState.guessing==0 ) {
						params.add(pd);
					}
				}
				else {
					break _loop103;
				}
				
			} while (true);
			}
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return params;
	}
	
	public final void returnTypeBrackersOnEndOfMethodHead() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop99:
		do {
			if ((LA(1)==LBRACK)) {
				match(LBRACK);
				match(RBRACK);
			}
			else {
				break _loop99;
			}
			
		} while (true);
		}
	}
	
	public final Set  throwsClause() throws RecognitionException, TokenStreamException {
		Set exceptions;
		
		
		exceptions = new LinkedHashSet();
		Token id, id2;
		
		
		match(LITERAL_throws);
		id=identifier();
		if ( inputState.guessing==0 ) {
			exceptions.add(id.getText());
		}
		{
		_loop96:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				id2=identifier();
				if ( inputState.guessing==0 ) {
					exceptions.add(id2.getText());
				}
			}
			else {
				break _loop96;
			}
			
		} while (true);
		}
		return exceptions;
	}
	
/** Declaration of a variable.  This can be a class/instance variable,
 *   or a local variable in a method
 * It can also include possible initialization.
 */
	public final void variableDeclarator() throws RecognitionException, TokenStreamException {
		
		
		match(IDENT);
		declaratorBrackets();
		varInitializer();
	}
	
	public final void declaratorBrackets() throws RecognitionException, TokenStreamException {
		
		Token  lb = null;
		
		{
		_loop82:
		do {
			if ((LA(1)==LBRACK)) {
				lb = LT(1);
				match(LBRACK);
				match(RBRACK);
			}
			else {
				break _loop82;
			}
			
		} while (true);
		}
	}
	
	public final void varInitializer() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case ASSIGN:
		{
			match(ASSIGN);
			initializer();
			break;
		}
		case SEMI:
		case COMMA:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void initializer() throws RecognitionException, TokenStreamException {
		
		
		switch ( LA(1)) {
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			expression();
			break;
		}
		case LCURLY:
		{
			arrayInitializer();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void arrayInitializer() throws RecognitionException, TokenStreamException {
		
		
		match(LCURLY);
		{
		switch ( LA(1)) {
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LCURLY:
		case LPAREN:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			initializer();
			{
			_loop88:
			do {
				if ((LA(1)==COMMA) && (_tokenSet_9.member(LA(2)))) {
					match(COMMA);
					initializer();
				}
				else {
					break _loop88;
				}
				
			} while (true);
			}
			{
			switch ( LA(1)) {
			case COMMA:
			{
				match(COMMA);
				break;
			}
			case RCURLY:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case RCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(RCURLY);
	}
	
/**
   the mother of all expressions

<pre>
 expressions
 Note that most of these expressions follow the pattern
   thisLevelExpression :
       nextHigherPrecedenceExpression
           (OPERATOR nextHigherPrecedenceExpression)*
 which is a standard recursive definition for a parsing an expression.
 The operators in java have the following precedences:
    lowest  (13)  = *= /= %= += -= <<= >>= >>>= &= ^= |=
            (12)  ?:
            (11)  ||
            (10)  &&
            ( 9)  |
            ( 8)  ^
            ( 7)  &
            ( 6)  == !=
            ( 5)  < <= > >=
            ( 4)  << >>
            ( 3)  +(binary) -(binary)
            ( 2)  * / %
            ( 1)  ++ -- +(unary) -(unary)  ~  !  (type)
                  []   () (method call)  . (dot -- identifier qualification)
                  new   ()  (explicit parenthesis)

 the last two are not usually on a precedence chart; I put them in
 to point out that new has a higher precedence than '.', so you
 can validy use
     new Frame().show()
 
 Note that the above precedence levels map to the rules below...
 Once you have a precedence chart, writing the appropriate rules as below
   is usually very straightfoward
</pre>



**/
	public final void expression() throws RecognitionException, TokenStreamException {
		
		
		assignmentExpression();
	}
	
	public final StringPair  parameterDeclaration() throws RecognitionException, TokenStreamException {
		StringPair sp;
		
		Token  name = null;
		
		StringBuffer brackets = null;
		Token type;
		sp = null;
		
		
		parameterModifier();
		type=typeSpec();
		name = LT(1);
		match(IDENT);
		brackets=parameterDeclaratorBrackets();
		if ( inputState.guessing==0 ) {
			
			String typeText = type.getText();
			if(brackets.length() > 0) {
			typeText += brackets.toString();
			}
			sp = new StringPair(typeText, name.getText());
			
		}
		return sp;
	}
	
	public final void parameterModifier() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case FINAL:
		{
			match(FINAL);
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final StringBuffer  parameterDeclaratorBrackets() throws RecognitionException, TokenStreamException {
		StringBuffer text;
		
		Token  lb = null;
		Token  rb = null;
		
		text = new StringBuffer();
		
		
		{
		_loop107:
		do {
			if ((LA(1)==LBRACK)) {
				lb = LT(1);
				match(LBRACK);
				rb = LT(1);
				match(RBRACK);
				if ( inputState.guessing==0 ) {
					text.append("[]");
				}
			}
			else {
				break _loop107;
			}
			
		} while (true);
		}
		return text;
	}
	
/**
   Handle assert or invariant conditions, if an invariant token is seen, clear
   the asserts, if an assert is seen, clear the invariants.  This should keep
   us out of trouble.
**/
	public final void assertOrInvariantCondition() throws RecognitionException, TokenStreamException {
		
		Token  assertCondition = null;
		Token  iv = null;
		Token  jdc = null;
		List assertTokens = new LinkedList();
		
		{
		match(JAVADOC_OPEN);
		{
		_loop113:
		do {
			switch ( LA(1)) {
			case ASSERT_CONDITION:
			{
				assertCondition = LT(1);
				match(ASSERT_CONDITION);
				if ( inputState.guessing==0 ) {
					assertTokens.add(assertCondition); clearInvariants();
				}
				break;
			}
			case PRE_CONDITION:
			{
				match(PRE_CONDITION);
				break;
			}
			case POST_CONDITION:
			{
				match(POST_CONDITION);
				break;
			}
			case INVARIANT_CONDITION:
			{
				iv = LT(1);
				match(INVARIANT_CONDITION);
				if ( inputState.guessing==0 ) {
					addInvariant(iv); assertTokens = new LinkedList();
				}
				break;
			}
			default:
			{
				break _loop113;
			}
			}
		} while (true);
		}
		jdc = LT(1);
		match(JAVADOC_CLOSE);
		if ( inputState.guessing==0 ) {
			addAsserts(assertTokens, jdc);
		}
		}
	}
	
	public final void statement() throws RecognitionException, TokenStreamException {
		
		Token  ret = null;
		Token  semi = null;
		
		CodePointPair dummyStartEnd;
		
		
		switch ( LA(1)) {
		case LCURLY:
		{
			dummyStartEnd=compoundStatement();
			break;
		}
		case LITERAL_class:
		{
			classDefinition();
			break;
		}
		case LITERAL_if:
		{
			match(LITERAL_if);
			match(LPAREN);
			expression();
			match(RPAREN);
			statement();
			{
			if ((LA(1)==LITERAL_else) && (_tokenSet_8.member(LA(2)))) {
				match(LITERAL_else);
				statement();
			}
			else if ((_tokenSet_10.member(LA(1))) && (_tokenSet_11.member(LA(2)))) {
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			break;
		}
		case LITERAL_for:
		{
			match(LITERAL_for);
			match(LPAREN);
			forInit();
			match(SEMI);
			forCond();
			match(SEMI);
			forIter();
			match(RPAREN);
			statement();
			break;
		}
		case LITERAL_while:
		{
			match(LITERAL_while);
			match(LPAREN);
			expression();
			match(RPAREN);
			statement();
			break;
		}
		case LITERAL_do:
		{
			match(LITERAL_do);
			statement();
			match(LITERAL_while);
			match(LPAREN);
			expression();
			match(RPAREN);
			match(SEMI);
			break;
		}
		case LITERAL_break:
		{
			match(LITERAL_break);
			{
			switch ( LA(1)) {
			case IDENT:
			{
				match(IDENT);
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(SEMI);
			break;
		}
		case LITERAL_continue:
		{
			match(LITERAL_continue);
			{
			switch ( LA(1)) {
			case IDENT:
			{
				match(IDENT);
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(SEMI);
			break;
		}
		case LITERAL_return:
		{
			ret = LT(1);
			match(LITERAL_return);
			{
			switch ( LA(1)) {
			case LITERAL_void:
			case LITERAL_boolean:
			case LITERAL_byte:
			case LITERAL_char:
			case LITERAL_short:
			case LITERAL_int:
			case LITERAL_float:
			case LITERAL_long:
			case LITERAL_double:
			case IDENT:
			case LPAREN:
			case PLUS:
			case MINUS:
			case INC:
			case DEC:
			case BNOT:
			case LNOT:
			case LITERAL_this:
			case LITERAL_super:
			case LITERAL_true:
			case LITERAL_false:
			case LITERAL_null:
			case LITERAL_new:
			case NUM_INT:
			case CHAR_LITERAL:
			case STRING_LITERAL:
			case NUM_FLOAT:
			{
				expression();
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			semi = LT(1);
			match(SEMI);
			if ( inputState.guessing==0 ) {
				
				//keep track of these points for post conditions
				CodePoint retcp = new CodePoint(ret.getLine(), ret.getColumn()-1);
				//add 1 so that code is inserted after the semi colon
				CodePoint semicp = new CodePoint(semi.getLine(), semi.getColumn());
				getSymtab().getCurrentMethod().addExit(new CodePointPair(retcp, semicp));
				
			}
			break;
		}
		case LITERAL_switch:
		{
			match(LITERAL_switch);
			match(LPAREN);
			expression();
			match(RPAREN);
			match(LCURLY);
			{
			_loop126:
			do {
				if ((LA(1)==LITERAL_case||LA(1)==LITERAL_default)) {
					casesGroup();
				}
				else {
					break _loop126;
				}
				
			} while (true);
			}
			match(RCURLY);
			break;
		}
		case LITERAL_try:
		{
			tryBlock();
			break;
		}
		case LITERAL_throw:
		{
			match(LITERAL_throw);
			expression();
			match(SEMI);
			break;
		}
		case SEMI:
		{
			match(SEMI);
			break;
		}
		case JAVADOC_OPEN:
		{
			assertOrInvariantCondition();
			break;
		}
		default:
			if ((LA(1)==FINAL) && (LA(2)==LITERAL_class)) {
				match(FINAL);
				classDefinition();
			}
			else if ((LA(1)==ABSTRACT) && (LA(2)==LITERAL_class)) {
				match(ABSTRACT);
				classDefinition();
			}
			else {
				boolean synPredMatched120 = false;
				if (((_tokenSet_12.member(LA(1))) && (_tokenSet_13.member(LA(2))))) {
					int _m120 = mark();
					synPredMatched120 = true;
					inputState.guessing++;
					try {
						{
						declaration();
						}
					}
					catch (RecognitionException pe) {
						synPredMatched120 = false;
					}
					rewind(_m120);
					inputState.guessing--;
				}
				if ( synPredMatched120 ) {
					declaration();
					match(SEMI);
				}
				else if ((_tokenSet_14.member(LA(1))) && (_tokenSet_15.member(LA(2)))) {
					expression();
					match(SEMI);
				}
				else if ((LA(1)==IDENT) && (LA(2)==COLON)) {
					match(IDENT);
					match(COLON);
					statement();
				}
				else if ((LA(1)==LITERAL_synchronized) && (LA(2)==LPAREN)) {
					match(LITERAL_synchronized);
					match(LPAREN);
					expression();
					match(RPAREN);
					dummyStartEnd=compoundStatement();
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}}
		}
		
	public final void forInit() throws RecognitionException, TokenStreamException {
		
		
		{
		boolean synPredMatched138 = false;
		if (((_tokenSet_12.member(LA(1))) && (_tokenSet_13.member(LA(2))))) {
			int _m138 = mark();
			synPredMatched138 = true;
			inputState.guessing++;
			try {
				{
				declaration();
				}
			}
			catch (RecognitionException pe) {
				synPredMatched138 = false;
			}
			rewind(_m138);
			inputState.guessing--;
		}
		if ( synPredMatched138 ) {
			declaration();
		}
		else if ((_tokenSet_14.member(LA(1))) && (_tokenSet_16.member(LA(2)))) {
			expressionList();
		}
		else if ((LA(1)==SEMI)) {
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
	}
	
	public final void forCond() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			expression();
			break;
		}
		case SEMI:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void forIter() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			expressionList();
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void casesGroup() throws RecognitionException, TokenStreamException {
		
		
		{
		int _cnt129=0;
		_loop129:
		do {
			if ((LA(1)==LITERAL_case||LA(1)==LITERAL_default) && (_tokenSet_17.member(LA(2)))) {
				aCase();
			}
			else {
				if ( _cnt129>=1 ) { break _loop129; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt129++;
		} while (true);
		}
		caseSList();
	}
	
	public final void tryBlock() throws RecognitionException, TokenStreamException {
		
		
		CodePointPair dummyStartEnd;
		
		
		match(LITERAL_try);
		dummyStartEnd=compoundStatement();
		{
		_loop145:
		do {
			if ((LA(1)==LITERAL_catch)) {
				handler();
			}
			else {
				break _loop145;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case LITERAL_finally:
		{
			match(LITERAL_finally);
			dummyStartEnd=compoundStatement();
			break;
		}
		case FINAL:
		case ABSTRACT:
		case JAVADOC_OPEN:
		case SEMI:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LITERAL_private:
		case LITERAL_public:
		case LITERAL_protected:
		case LITERAL_static:
		case LITERAL_transient:
		case LITERAL_native:
		case LITERAL_threadsafe:
		case LITERAL_synchronized:
		case LITERAL_volatile:
		case LITERAL_class:
		case LCURLY:
		case RCURLY:
		case LPAREN:
		case LITERAL_if:
		case LITERAL_else:
		case LITERAL_for:
		case LITERAL_while:
		case LITERAL_do:
		case LITERAL_break:
		case LITERAL_continue:
		case LITERAL_return:
		case LITERAL_switch:
		case LITERAL_throw:
		case LITERAL_case:
		case LITERAL_default:
		case LITERAL_try:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void aCase() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case LITERAL_case:
		{
			match(LITERAL_case);
			expression();
			break;
		}
		case LITERAL_default:
		{
			match(LITERAL_default);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(COLON);
	}
	
	public final void caseSList() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop134:
		do {
			if ((_tokenSet_8.member(LA(1)))) {
				statement();
			}
			else {
				break _loop134;
			}
			
		} while (true);
		}
	}
	
/**
   This is a list of expressions.
**/
	public final void expressionList() throws RecognitionException, TokenStreamException {
		
		
		expression();
		{
		_loop151:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				expression();
			}
			else {
				break _loop151;
			}
			
		} while (true);
		}
	}
	
	public final void handler() throws RecognitionException, TokenStreamException {
		
		
		CodePointPair dummyStartEnd;
		StringPair dummyParams;
		
		
		match(LITERAL_catch);
		match(LPAREN);
		dummyParams=parameterDeclaration();
		match(RPAREN);
		dummyStartEnd=compoundStatement();
	}
	
	public final void assignmentExpression() throws RecognitionException, TokenStreamException {
		
		
		conditionalExpression();
		{
		switch ( LA(1)) {
		case ASSIGN:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		{
			{
			switch ( LA(1)) {
			case ASSIGN:
			{
				match(ASSIGN);
				break;
			}
			case PLUS_ASSIGN:
			{
				match(PLUS_ASSIGN);
				break;
			}
			case MINUS_ASSIGN:
			{
				match(MINUS_ASSIGN);
				break;
			}
			case STAR_ASSIGN:
			{
				match(STAR_ASSIGN);
				break;
			}
			case DIV_ASSIGN:
			{
				match(DIV_ASSIGN);
				break;
			}
			case MOD_ASSIGN:
			{
				match(MOD_ASSIGN);
				break;
			}
			case SR_ASSIGN:
			{
				match(SR_ASSIGN);
				break;
			}
			case BSR_ASSIGN:
			{
				match(BSR_ASSIGN);
				break;
			}
			case SL_ASSIGN:
			{
				match(SL_ASSIGN);
				break;
			}
			case BAND_ASSIGN:
			{
				match(BAND_ASSIGN);
				break;
			}
			case BXOR_ASSIGN:
			{
				match(BXOR_ASSIGN);
				break;
			}
			case BOR_ASSIGN:
			{
				match(BOR_ASSIGN);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			assignmentExpression();
			break;
		}
		case SEMI:
		case RBRACK:
		case RCURLY:
		case COMMA:
		case RPAREN:
		case COLON:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void conditionalExpression() throws RecognitionException, TokenStreamException {
		
		
		logicalOrExpression();
		{
		switch ( LA(1)) {
		case QUESTION:
		{
			match(QUESTION);
			conditionalExpression();
			match(COLON);
			conditionalExpression();
			break;
		}
		case SEMI:
		case RBRACK:
		case RCURLY:
		case COMMA:
		case RPAREN:
		case ASSIGN:
		case COLON:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
/**
   logical or (||)  (level 11)

**/
	public final void logicalOrExpression() throws RecognitionException, TokenStreamException {
		
		
		logicalAndExpression();
		{
		_loop159:
		do {
			if ((LA(1)==LOR)) {
				match(LOR);
				logicalAndExpression();
			}
			else {
				break _loop159;
			}
			
		} while (true);
		}
	}
	
/**
   logical and (&&)  (level 10)

**/
	public final void logicalAndExpression() throws RecognitionException, TokenStreamException {
		
		
		inclusiveOrExpression();
		{
		_loop162:
		do {
			if ((LA(1)==LAND)) {
				match(LAND);
				inclusiveOrExpression();
			}
			else {
				break _loop162;
			}
			
		} while (true);
		}
	}
	
/**
   bitwise or non-short-circuiting or (|)  (level 9)

**/
	public final void inclusiveOrExpression() throws RecognitionException, TokenStreamException {
		
		
		exclusiveOrExpression();
		{
		_loop165:
		do {
			if ((LA(1)==BOR)) {
				match(BOR);
				exclusiveOrExpression();
			}
			else {
				break _loop165;
			}
			
		} while (true);
		}
	}
	
/**
   exclusive or (^)  (level 8)
**/
	public final void exclusiveOrExpression() throws RecognitionException, TokenStreamException {
		
		
		andExpression();
		{
		_loop168:
		do {
			if ((LA(1)==BXOR)) {
				match(BXOR);
				andExpression();
			}
			else {
				break _loop168;
			}
			
		} while (true);
		}
	}
	
/**
   bitwise or non-short-circuiting and (&)  (level 7)

**/
	public final void andExpression() throws RecognitionException, TokenStreamException {
		
		
		equalityExpression();
		{
		_loop171:
		do {
			if ((LA(1)==BAND)) {
				match(BAND);
				equalityExpression();
			}
			else {
				break _loop171;
			}
			
		} while (true);
		}
	}
	
/**
   equality/inequality (==/!=) (level 6)

**/
	public final void equalityExpression() throws RecognitionException, TokenStreamException {
		
		
		relationalExpression();
		{
		_loop175:
		do {
			if ((LA(1)==NOT_EQUAL||LA(1)==EQUAL)) {
				{
				switch ( LA(1)) {
				case NOT_EQUAL:
				{
					match(NOT_EQUAL);
					break;
				}
				case EQUAL:
				{
					match(EQUAL);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				relationalExpression();
			}
			else {
				break _loop175;
			}
			
		} while (true);
		}
	}
	
/**
   boolean relational expressions (level 5)

**/
	public final void relationalExpression() throws RecognitionException, TokenStreamException {
		
		
		Token dummyType;
		
		
		shiftExpression();
		{
		switch ( LA(1)) {
		case SEMI:
		case RBRACK:
		case RCURLY:
		case COMMA:
		case RPAREN:
		case ASSIGN:
		case COLON:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		case QUESTION:
		case LOR:
		case LAND:
		case BOR:
		case BXOR:
		case BAND:
		case NOT_EQUAL:
		case EQUAL:
		case LT:
		case GT:
		case LE:
		case GE:
		{
			{
			_loop180:
			do {
				if (((LA(1) >= LT && LA(1) <= GE))) {
					{
					switch ( LA(1)) {
					case LT:
					{
						match(LT);
						break;
					}
					case GT:
					{
						match(GT);
						break;
					}
					case LE:
					{
						match(LE);
						break;
					}
					case GE:
					{
						match(GE);
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					shiftExpression();
				}
				else {
					break _loop180;
				}
				
			} while (true);
			}
			break;
		}
		case LITERAL_instanceof:
		{
			match(LITERAL_instanceof);
			dummyType=typeSpec();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
/**
   bit shift expressions (level 4)

**/
	public final void shiftExpression() throws RecognitionException, TokenStreamException {
		
		
		additiveExpression();
		{
		_loop184:
		do {
			if (((LA(1) >= SL && LA(1) <= BSR))) {
				{
				switch ( LA(1)) {
				case SL:
				{
					match(SL);
					break;
				}
				case SR:
				{
					match(SR);
					break;
				}
				case BSR:
				{
					match(BSR);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				additiveExpression();
			}
			else {
				break _loop184;
			}
			
		} while (true);
		}
	}
	
/**
   binary addition/subtraction (level 3)
**/
	public final void additiveExpression() throws RecognitionException, TokenStreamException {
		
		
		multiplicativeExpression();
		{
		_loop188:
		do {
			if ((LA(1)==PLUS||LA(1)==MINUS)) {
				{
				switch ( LA(1)) {
				case PLUS:
				{
					match(PLUS);
					break;
				}
				case MINUS:
				{
					match(MINUS);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				multiplicativeExpression();
			}
			else {
				break _loop188;
			}
			
		} while (true);
		}
	}
	
/**
   multiplication/division/modulo (level 2)

**/
	public final void multiplicativeExpression() throws RecognitionException, TokenStreamException {
		
		
		unaryExpression();
		{
		_loop192:
		do {
			if ((_tokenSet_18.member(LA(1)))) {
				{
				switch ( LA(1)) {
				case STAR:
				{
					match(STAR);
					break;
				}
				case DIV:
				{
					match(DIV);
					break;
				}
				case MOD:
				{
					match(MOD);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				unaryExpression();
			}
			else {
				break _loop192;
			}
			
		} while (true);
		}
	}
	
	public final void unaryExpression() throws RecognitionException, TokenStreamException {
		
		
		switch ( LA(1)) {
		case INC:
		{
			match(INC);
			unaryExpression();
			break;
		}
		case DEC:
		{
			match(DEC);
			unaryExpression();
			break;
		}
		case MINUS:
		{
			match(MINUS);
			unaryExpression();
			break;
		}
		case PLUS:
		{
			match(PLUS);
			unaryExpression();
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			unaryExpressionNotPlusMinus();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void unaryExpressionNotPlusMinus() throws RecognitionException, TokenStreamException {
		
		
		Token dummyToken;
		
		
		switch ( LA(1)) {
		case BNOT:
		{
			match(BNOT);
			unaryExpression();
			break;
		}
		case LNOT:
		{
			match(LNOT);
			unaryExpression();
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			{
			if ((LA(1)==LPAREN) && ((LA(2) >= LITERAL_void && LA(2) <= LITERAL_double))) {
				match(LPAREN);
				dummyToken=builtInTypeSpec();
				match(RPAREN);
				unaryExpression();
			}
			else {
				boolean synPredMatched197 = false;
				if (((LA(1)==LPAREN) && (LA(2)==IDENT))) {
					int _m197 = mark();
					synPredMatched197 = true;
					inputState.guessing++;
					try {
						{
						match(LPAREN);
						classTypeSpec();
						match(RPAREN);
						unaryExpressionNotPlusMinus();
						}
					}
					catch (RecognitionException pe) {
						synPredMatched197 = false;
					}
					rewind(_m197);
					inputState.guessing--;
				}
				if ( synPredMatched197 ) {
					match(LPAREN);
					dummyToken=classTypeSpec();
					match(RPAREN);
					unaryExpressionNotPlusMinus();
				}
				else if ((_tokenSet_19.member(LA(1))) && (_tokenSet_20.member(LA(2)))) {
					postfixExpression();
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		
/**
   qualified names, array expressions, method invocation, post inc/dec

**/
	public final void postfixExpression() throws RecognitionException, TokenStreamException {
		
		Token id;
		
		switch ( LA(1)) {
		case IDENT:
		case LPAREN:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			primaryExpression();
			{
			_loop204:
			do {
				switch ( LA(1)) {
				case DOT:
				{
					match(DOT);
					{
					switch ( LA(1)) {
					case IDENT:
					{
						match(IDENT);
						break;
					}
					case LITERAL_this:
					{
						match(LITERAL_this);
						break;
					}
					case LITERAL_class:
					{
						match(LITERAL_class);
						break;
					}
					case LITERAL_new:
					{
						newExpression();
						break;
					}
					case LITERAL_super:
					{
						match(LITERAL_super);
						match(LPAREN);
						{
						switch ( LA(1)) {
						case LITERAL_void:
						case LITERAL_boolean:
						case LITERAL_byte:
						case LITERAL_char:
						case LITERAL_short:
						case LITERAL_int:
						case LITERAL_float:
						case LITERAL_long:
						case LITERAL_double:
						case IDENT:
						case LPAREN:
						case PLUS:
						case MINUS:
						case INC:
						case DEC:
						case BNOT:
						case LNOT:
						case LITERAL_this:
						case LITERAL_super:
						case LITERAL_true:
						case LITERAL_false:
						case LITERAL_null:
						case LITERAL_new:
						case NUM_INT:
						case CHAR_LITERAL:
						case STRING_LITERAL:
						case NUM_FLOAT:
						{
							expressionList();
							break;
						}
						case RPAREN:
						{
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						match(RPAREN);
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					break;
				}
				case LPAREN:
				{
					match(LPAREN);
					argList();
					match(RPAREN);
					break;
				}
				default:
					if ((LA(1)==LBRACK) && (LA(2)==RBRACK)) {
						{
						int _cnt203=0;
						_loop203:
						do {
							if ((LA(1)==LBRACK)) {
								match(LBRACK);
								match(RBRACK);
							}
							else {
								if ( _cnt203>=1 ) { break _loop203; } else {throw new NoViableAltException(LT(1), getFilename());}
							}
							
							_cnt203++;
						} while (true);
						}
						match(DOT);
						match(LITERAL_class);
					}
					else if ((LA(1)==LBRACK) && (_tokenSet_14.member(LA(2)))) {
						match(LBRACK);
						expression();
						match(RBRACK);
					}
				else {
					break _loop204;
				}
				}
			} while (true);
			}
			{
			switch ( LA(1)) {
			case INC:
			{
				match(INC);
				break;
			}
			case DEC:
			{
				match(DEC);
				break;
			}
			case SEMI:
			case RBRACK:
			case STAR:
			case RCURLY:
			case COMMA:
			case RPAREN:
			case ASSIGN:
			case COLON:
			case PLUS_ASSIGN:
			case MINUS_ASSIGN:
			case STAR_ASSIGN:
			case DIV_ASSIGN:
			case MOD_ASSIGN:
			case SR_ASSIGN:
			case BSR_ASSIGN:
			case SL_ASSIGN:
			case BAND_ASSIGN:
			case BXOR_ASSIGN:
			case BOR_ASSIGN:
			case QUESTION:
			case LOR:
			case LAND:
			case BOR:
			case BXOR:
			case BAND:
			case NOT_EQUAL:
			case EQUAL:
			case LT:
			case GT:
			case LE:
			case GE:
			case LITERAL_instanceof:
			case SL:
			case SR:
			case BSR:
			case PLUS:
			case MINUS:
			case DIV:
			case MOD:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			id=builtInType();
			{
			_loop207:
			do {
				if ((LA(1)==LBRACK)) {
					match(LBRACK);
					match(RBRACK);
				}
				else {
					break _loop207;
				}
				
			} while (true);
			}
			match(DOT);
			match(LITERAL_class);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
/**
   the basic element of an expression
**/
	public final void primaryExpression() throws RecognitionException, TokenStreamException {
		
		
		switch ( LA(1)) {
		case IDENT:
		{
			match(IDENT);
			break;
		}
		case LITERAL_new:
		{
			newExpression();
			break;
		}
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			constant();
			break;
		}
		case LITERAL_super:
		{
			match(LITERAL_super);
			break;
		}
		case LITERAL_true:
		{
			match(LITERAL_true);
			break;
		}
		case LITERAL_false:
		{
			match(LITERAL_false);
			break;
		}
		case LITERAL_this:
		{
			match(LITERAL_this);
			break;
		}
		case LITERAL_null:
		{
			match(LITERAL_null);
			break;
		}
		case LPAREN:
		{
			match(LPAREN);
			assignmentExpression();
			match(RPAREN);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
/**
   object instantiation.
**/
	public final void newExpression() throws RecognitionException, TokenStreamException {
		
		Token t;
		
		match(LITERAL_new);
		t=type();
		{
		switch ( LA(1)) {
		case LPAREN:
		{
			match(LPAREN);
			argList();
			match(RPAREN);
			{
			switch ( LA(1)) {
			case LCURLY:
			{
				classBlock(null, false, true, t);
				break;
			}
			case SEMI:
			case LBRACK:
			case RBRACK:
			case DOT:
			case STAR:
			case RCURLY:
			case COMMA:
			case LPAREN:
			case RPAREN:
			case ASSIGN:
			case COLON:
			case PLUS_ASSIGN:
			case MINUS_ASSIGN:
			case STAR_ASSIGN:
			case DIV_ASSIGN:
			case MOD_ASSIGN:
			case SR_ASSIGN:
			case BSR_ASSIGN:
			case SL_ASSIGN:
			case BAND_ASSIGN:
			case BXOR_ASSIGN:
			case BOR_ASSIGN:
			case QUESTION:
			case LOR:
			case LAND:
			case BOR:
			case BXOR:
			case BAND:
			case NOT_EQUAL:
			case EQUAL:
			case LT:
			case GT:
			case LE:
			case GE:
			case LITERAL_instanceof:
			case SL:
			case SR:
			case BSR:
			case PLUS:
			case MINUS:
			case DIV:
			case MOD:
			case INC:
			case DEC:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case LBRACK:
		{
			newArrayDeclarator();
			{
			switch ( LA(1)) {
			case LCURLY:
			{
				arrayInitializer();
				break;
			}
			case SEMI:
			case LBRACK:
			case RBRACK:
			case DOT:
			case STAR:
			case RCURLY:
			case COMMA:
			case LPAREN:
			case RPAREN:
			case ASSIGN:
			case COLON:
			case PLUS_ASSIGN:
			case MINUS_ASSIGN:
			case STAR_ASSIGN:
			case DIV_ASSIGN:
			case MOD_ASSIGN:
			case SR_ASSIGN:
			case BSR_ASSIGN:
			case SL_ASSIGN:
			case BAND_ASSIGN:
			case BXOR_ASSIGN:
			case BOR_ASSIGN:
			case QUESTION:
			case LOR:
			case LAND:
			case BOR:
			case BXOR:
			case BAND:
			case NOT_EQUAL:
			case EQUAL:
			case LT:
			case GT:
			case LE:
			case GE:
			case LITERAL_instanceof:
			case SL:
			case SR:
			case BSR:
			case PLUS:
			case MINUS:
			case DIV:
			case MOD:
			case INC:
			case DEC:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void argList() throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case IDENT:
		case LPAREN:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		{
			expressionList();
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void constant() throws RecognitionException, TokenStreamException {
		
		
		switch ( LA(1)) {
		case NUM_INT:
		{
			match(NUM_INT);
			break;
		}
		case CHAR_LITERAL:
		{
			match(CHAR_LITERAL);
			break;
		}
		case STRING_LITERAL:
		{
			match(STRING_LITERAL);
			break;
		}
		case NUM_FLOAT:
		{
			match(NUM_FLOAT);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void newArrayDeclarator() throws RecognitionException, TokenStreamException {
		
		
		{
		int _cnt218=0;
		_loop218:
		do {
			if ((LA(1)==LBRACK) && (_tokenSet_21.member(LA(2)))) {
				match(LBRACK);
				{
				switch ( LA(1)) {
				case LITERAL_void:
				case LITERAL_boolean:
				case LITERAL_byte:
				case LITERAL_char:
				case LITERAL_short:
				case LITERAL_int:
				case LITERAL_float:
				case LITERAL_long:
				case LITERAL_double:
				case IDENT:
				case LPAREN:
				case PLUS:
				case MINUS:
				case INC:
				case DEC:
				case BNOT:
				case LNOT:
				case LITERAL_this:
				case LITERAL_super:
				case LITERAL_true:
				case LITERAL_false:
				case LITERAL_null:
				case LITERAL_new:
				case NUM_INT:
				case CHAR_LITERAL:
				case STRING_LITERAL:
				case NUM_FLOAT:
				{
					expression();
					break;
				}
				case RBRACK:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				match(RBRACK);
			}
			else {
				if ( _cnt218>=1 ) { break _loop218; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt218++;
		} while (true);
		}
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"BLOCK",
		"MODIFIERS",
		"OBJBLOCK",
		"SLIST",
		"CTOR_DEF",
		"METHOD_DEF",
		"VARIABLE_DEF",
		"INSTANCE_INIT",
		"STATIC_INIT",
		"TYPE",
		"CLASS_DEF",
		"INTERFACE_DEF",
		"PACKAGE_DEF",
		"ARRAY_DECLARATOR",
		"EXTENDS_CLAUSE",
		"IMPLEMENTS_CLAUSE",
		"PARAMETERS",
		"PARAMETER_DEF",
		"LABELED_STAT",
		"TYPECAST",
		"INDEX_OP",
		"POST_INC",
		"POST_DEC",
		"METHOD_CALL",
		"EXPR",
		"ARRAY_INIT",
		"IMPORT",
		"UNARY_MINUS",
		"UNARY_PLUS",
		"CASE_GROUP",
		"ELIST",
		"FOR_INIT",
		"FOR_CONDITION",
		"FOR_ITERATOR",
		"EMPTY_STAT",
		"\"final\"",
		"\"abstract\"",
		"JAVADOC_OPEN",
		"JAVADOC_CLOSE",
		"POST_CONDITION",
		"PRE_CONDITION",
		"ASSERT_CONDITION",
		"INVARIANT_CONDITION",
		"CONDITION",
		"MESSAGE",
		"\"package\"",
		"SEMI",
		"\"import\"",
		"LBRACK",
		"RBRACK",
		"\"void\"",
		"\"boolean\"",
		"\"byte\"",
		"\"char\"",
		"\"short\"",
		"\"int\"",
		"\"float\"",
		"\"long\"",
		"\"double\"",
		"IDENT",
		"DOT",
		"STAR",
		"\"private\"",
		"\"public\"",
		"\"protected\"",
		"\"static\"",
		"\"transient\"",
		"\"native\"",
		"\"threadsafe\"",
		"\"synchronized\"",
		"\"volatile\"",
		"\"class\"",
		"\"extends\"",
		"\"interface\"",
		"LCURLY",
		"RCURLY",
		"COMMA",
		"\"implements\"",
		"LPAREN",
		"RPAREN",
		"ASSIGN",
		"\"throws\"",
		"COLON",
		"\"if\"",
		"\"else\"",
		"\"for\"",
		"\"while\"",
		"\"do\"",
		"\"break\"",
		"\"continue\"",
		"\"return\"",
		"\"switch\"",
		"\"throw\"",
		"\"case\"",
		"\"default\"",
		"\"try\"",
		"\"finally\"",
		"\"catch\"",
		"PLUS_ASSIGN",
		"MINUS_ASSIGN",
		"STAR_ASSIGN",
		"DIV_ASSIGN",
		"MOD_ASSIGN",
		"SR_ASSIGN",
		"BSR_ASSIGN",
		"SL_ASSIGN",
		"BAND_ASSIGN",
		"BXOR_ASSIGN",
		"BOR_ASSIGN",
		"QUESTION",
		"LOR",
		"LAND",
		"BOR",
		"BXOR",
		"BAND",
		"NOT_EQUAL",
		"EQUAL",
		"LT",
		"GT",
		"LE",
		"GE",
		"\"instanceof\"",
		"SL",
		"SR",
		"BSR",
		"PLUS",
		"MINUS",
		"DIV",
		"MOD",
		"INC",
		"DEC",
		"BNOT",
		"LNOT",
		"\"this\"",
		"\"super\"",
		"\"true\"",
		"\"false\"",
		"\"null\"",
		"\"new\"",
		"NUM_INT",
		"CHAR_LITERAL",
		"STRING_LITERAL",
		"NUM_FLOAT",
		"WS",
		"SL_COMMENT",
		"ML_COMMENT",
		"ESC",
		"HEX_DIGIT",
		"VOCAB",
		"EXPONENT",
		"FLOAT_SUFFIX"
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { 1129748197539840L, 12284L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	private static final long[] mk_tokenSet_1() {
		long[] data = { 3944497964646402L, 12284L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_1 = new BitSet(mk_tokenSet_1());
	private static final long[] mk_tokenSet_2() {
		long[] data = { -9222105949215391742L, 12284L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_2 = new BitSet(mk_tokenSet_2());
	private static final long[] mk_tokenSet_3() {
		long[] data = { 1649267441664L, 2044L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_3 = new BitSet(mk_tokenSet_3());
	private static final long[] mk_tokenSet_4() {
		long[] data = { -18012749242040320L, 12284L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_4 = new BitSet(mk_tokenSet_4());
	private static final long[] mk_tokenSet_5() {
		long[] data = { -13509149614669824L, 274429L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_5 = new BitSet(mk_tokenSet_5());
	private static final long[] mk_tokenSet_6() {
		long[] data = { -9218868437227405312L, 1L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_6 = new BitSet(mk_tokenSet_6());
	private static final long[] mk_tokenSet_7() {
		long[] data = { 5629499534213120L, 1114112L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_7 = new BitSet(mk_tokenSet_7());
	private static final long[] mk_tokenSet_8() {
		long[] data = { -16884650311942144L, 42924789756L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_8 = new BitSet(mk_tokenSet_8());
	private static final long[] mk_tokenSet_9() {
		long[] data = { -18014398509481984L, 278528L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_9 = new BitSet(mk_tokenSet_9());
	private static final long[] mk_tokenSet_10() {
		long[] data = { -16884650311942144L, 68711403516L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_10 = new BitSet(mk_tokenSet_10());
	private static final long[] mk_tokenSet_11() {
		long[] data = { -12244711242727424L, -2822145L, 524287L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_11 = new BitSet(mk_tokenSet_11());
	private static final long[] mk_tokenSet_12() {
		long[] data = { -18012749242040320L, 2044L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_12 = new BitSet(mk_tokenSet_12());
	private static final long[] mk_tokenSet_13() {
		long[] data = { -13509149614669824L, 2045L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_13 = new BitSet(mk_tokenSet_13());
	private static final long[] mk_tokenSet_14() {
		long[] data = { -18014398509481984L, 262144L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_14 = new BitSet(mk_tokenSet_14());
	private static final long[] mk_tokenSet_15() {
		long[] data = { -12384898975268864L, -274876596221L, 524287L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_15 = new BitSet(mk_tokenSet_15());
	private static final long[] mk_tokenSet_16() {
		long[] data = { -12384898975268864L, -274876530685L, 524287L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_16 = new BitSet(mk_tokenSet_16());
	private static final long[] mk_tokenSet_17() {
		long[] data = { -18014398509481984L, 4456448L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_17 = new BitSet(mk_tokenSet_17());
	private static final long[] mk_tokenSet_18() {
		long[] data = { 0L, 2L, 24L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_18 = new BitSet(mk_tokenSet_18());
	private static final long[] mk_tokenSet_19() {
		long[] data = { -18014398509481984L, 262144L, 523776L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_19 = new BitSet(mk_tokenSet_19());
	private static final long[] mk_tokenSet_20() {
		long[] data = { -3377699720527872L, -274871779325L, 524287L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_20 = new BitSet(mk_tokenSet_20());
	private static final long[] mk_tokenSet_21() {
		long[] data = { -9007199254740992L, 262144L, 524262L, 0L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_21 = new BitSet(mk_tokenSet_21());
	
	}
