header {
/*
 * Copyright (c) 2000
 *      Jon Schewe.  All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * I'd appreciate comments/suggestions on the code jpschewe@mtu.net
 */
package net.mtu.eggplant.dbc;

import net.mtu.eggplant.util.StringPair;
import net.mtu.eggplant.util.Pair;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.LinkedList;
import java.util.LinkedHashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
}
class Java14Recognizer extends Parser;

options {
	k=2;
	codeGenMakeSwitchThreshold= 2;
	codeGenBitsetTestThreshold= 3;
	defaultErrorHandler= false;
	importVocab=Java;
}

{

  private static final Log LOG = LogFactory.getLog(Java14Recognizer.class);

// Define some helper methods, JPS copied in from java.g because antlr is being stupid

  public void print(final String s) {
    System.out.println("Parser: " + s);
  }

  /**
     set the symbol table object to use.
  **/
  public void setSymtab(final Symtab symtab) {
    _symtab = symtab;
  }

  private Symtab _symtab;

  /**
     Get the symbol table object being used.
  **/
  public Symtab getSymtab() {
    return _symtab;
  }
  
  /**
     Given the id Token from the implements clause find the package the
     interface is defined in.  Parse the interface for assertions and add
     an AssertInterface object to the current class so we can check against
     it later.
  **/
  private void parseImplementedInterface(final Token t) {
    String interfaceName = t.getText();
    String packageName = "";
    if(interfaceName.indexOf('.') > 0) {
      //already qualified interface, break up into package and name
      int lastDot = interfaceName.lastIndexOf('.');
      packageName = interfaceName.substring(0, lastDot);
      interfaceName = interfaceName.substring(lastDot+1);
    }
    else {
      //Need to figure out the right package here, search through the
      //current package, _imports
    }

    //actually parse the file so we can check
    //against it later, if no file can be found then don't add it to the
    //class, otherwise add it to the current class with
    //getSymtab().getCurrentClass().addInterface(assertInterface)

  }
  
  /**
     add an assert.  This should get cached with the file object?
     @param asserts asserts, ordered as they appear in the code. 
  **/
  private void addAsserts(final List asserts, final Token jdClose) {
    if(asserts != null && asserts.size() > 0) {
      int line = jdClose.getLine();
      int column = jdClose.getColumn()-1 + jdClose.getText().length();
      //System.out.println("adding assert"
      //  + " line: " + line
      //  + " column: " + column
      //  );
      StringBuffer codeFrag = new StringBuffer();
      Iterator iter = asserts.iterator();
      while(iter.hasNext()) {
        AssertToken assertToken = (AssertToken)iter.next();
        String code = CodeGenerator.generateAssertion(assertToken);
        codeFrag.append(code);
      }
      CodeFragment codeFragment = new CodeFragment(new CodePoint(line, column), codeFrag.toString(), CodeFragmentType.ASSERT);
      getSymtab().addCodeFragment(codeFragment);
    }
  }

  
  private List _invariants = new LinkedList();

  /**
     add invariants to this class

     @pre (invariant != null)
  **/
  private void addInvariant(final Token invariant) {
    if(! (invariant instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + invariant.getClass());
    }
    _invariants.add(invariant);
  }

  /**
     get the invariants for this class.
     Invariants are in the order they appear in the source.
  **/
  private List getInvariants() {
    return _invariants;
  }

  /**
     Clear the list of invariants for this class.
  **/
  private void clearInvariants() {
    _invariants = new LinkedList();
  }


  private List _preConditions = new LinkedList();
  /**
     Get the list of preconditions that have been seen since the last clear.
     Preconditions are in the order they appear in the source.
  **/
  public List getPreConditions() {
    return _preConditions;
  }

  /**
     Add a precondition to the list of preconditions.
  **/
  public void addPreCondition(final Token pre) {
    if(! (pre instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + pre.getClass());
    }
    _preConditions.add(pre);
  }

  /**
     clear out the list of preconditions.
  **/
  public void clearPreConditions() {
    _preConditions = new LinkedList();
  }

  private List _postConditions = new LinkedList();
  /**
     Get the list of postconditions that have been seen since the last clear.
     Postconditions are in the order they appear in the source.
  **/
  public List getPostConditions() {
    return _postConditions;
  }

  /**
     clear out the list of postconditions.
  **/
  public void addPostCondition(final Token post) {
    if(! (post instanceof AssertToken)) {
      throw new RuntimeException("Expecting AssertToken! " + post.getClass());
    }
    _postConditions.add(post);
  }

  /**
     clear out the list of postconditions.
  **/
  public void clearPostConditions() {
    _postConditions = new LinkedList();
  }    

}
statement {
  CodePointPair dummyStartEnd;
}
:// A list of statements in curly braces -- start a new scope!    
    dummyStartEnd=compoundStatement

    // class definition
    |   classDefinition

    // final class definition
    |   "final" classDefinition

    // abstract class definition
    |   "abstract" classDefinition

    // declarations are ambiguous with "ID DOT" relative to expression
    // statements.  Must backtrack to be sure.  Could use a semantic
    // predicate to test symbol table to see what the type was coming
    // up, but that's pretty hard without a symbol table ;)
    |   (declaration)=> declaration SEMI

    // An expression statement.  This could be a method call,
    // assignment statement, or any other expression evaluated for
    // side-effects.
    |   expression SEMI

    // Attach a label to the front of a statement
    |   IDENT COLON statement

    // If-else statement
    |   "if" LPAREN expression RPAREN statement
    (
        // CONFLICT: the old "dangling-else" problem...
        //           ANTLR generates proper code matching
        //           as soon as possible.  Hush warning.
        options {
        warnWhenFollowAmbig = false;
        }
    :
        "else" statement
    )?

    // For statement
    |   "for"
    LPAREN
    forInit SEMI   // initializer
    forCond SEMI   // condition test
    forIter         // updater
    RPAREN
    statement                     // statement to loop over

    // While statement
    |   "while" LPAREN expression RPAREN statement

    // do-while statement
    |   "do" statement "while" LPAREN expression RPAREN SEMI

    // get out of a loop (or switch)
    |   "break" (IDENT)? SEMI

    // do next iteration of a loop
    |   "continue" (IDENT)? SEMI

    // Return an expression
    |   ret:"return" (expression)? semi:SEMI
      {
    //keep track of these points for post conditions
    CodePoint retcp = new CodePoint(ret.getLine(), ret.getColumn()-1);
    //add 1 so that code is inserted after the semi colon
    CodePoint semicp = new CodePoint(semi.getLine(), semi.getColumn());
    getSymtab().getCurrentMethod().addExit(new CodePointPair(retcp, semicp));
      }

    // switch/case statement
    |   "switch" LPAREN expression RPAREN LCURLY
    ( casesGroup )*
    RCURLY

    // exception try-catch block
    |   tryBlock

    // throw an exception
    |   "throw" expression SEMI

    // synchronize a statement
    |   "synchronized" LPAREN expression RPAREN dummyStartEnd=compoundStatement

    // empty statement
    |   SEMI
      
      //assertion, checks invariants too for the class definitions allowed in statement
    |   assertOrInvariantCondition

    //JDK 1.4 assert keyword
    | "assert" expression (COLON expression)? SEMI

    ;

// inherited from grammar JavaRecognizer
unit :pre start middle end EOF ;

// inherited from grammar JavaRecognizer
pre :uselessComments ;

// inherited from grammar JavaRecognizer
start {
  String packageName = null;
}
:(packageName=packageDefinition uselessComments)?
    {//stuff to do after finding packageName
      getSymtab().setCurrentPackageName(packageName);
      LOG.debug("just found the package: " + packageName);
      //Now we just need to check to make sure the destination file is older
      if(!getSymtab().getConfiguration().ignoreTimeStamp()) {
        if(!getSymtab().isDestinationOlderThanCurrentFile(packageName)) {
          // already parsed this file and force is off, skip it
          throw new FileAlreadyParsedException();
        }
      }

      //Put the print here so you don't see it unless we're really parsing the files
      if(getSymtab().getConfiguration().isVerbose()) {
        System.out.println("  " + getSymtab().getCurrentFile().getFile().getAbsolutePath());
      }
    }
  ;

// inherited from grammar JavaRecognizer
middle :(importDefinition uselessComments)*
    {
      clearInvariants();
    }
  ;

// inherited from grammar JavaRecognizer
end :((invariantCondition)? typeDefinition)* ;

// inherited from grammar JavaRecognizer
uselessComments :(javadocComment) => ( (invariantCondition typeDefinition) => /* empty */ | javadocComment uselessComments )
  | /* empty */ ;

// inherited from grammar JavaRecognizer
javadocComment :JAVADOC_OPEN ( INVARIANT_CONDITION | PRE_CONDITION | POST_CONDITION | ASSERT_CONDITION )* JAVADOC_CLOSE
  ;

// inherited from grammar JavaRecognizer
invariantCondition :JAVADOC_OPEN ( iv:INVARIANT_CONDITION { addInvariant(iv); } | PRE_CONDITION | POST_CONDITION | ASSERT_CONDITION )* JAVADOC_CLOSE
  ;

// inherited from grammar JavaRecognizer
packageDefinition returns [String packageName]{
  Token id = null;
  packageName = null;
}
:"package" id=identifier SEMI
    {
      packageName = id.getText();
    }
  ;

// inherited from grammar JavaRecognizer
importDefinition :"import" identifierStar SEMI
  ;

// inherited from grammar JavaRecognizer
typeDefinition {
  Set dummyMods;
}
:dummyMods=modifiers
    ( classDefinition
    | interfaceDefinition
    )
  | SEMI
  ;

// inherited from grammar JavaRecognizer
declaration {
  Set dummyMods;
  Token dummyType;
}
:dummyMods=modifiers dummyType=typeSpec variableDefinitions
  ;

// inherited from grammar JavaRecognizer
modifiers returns [Set mods]{
  mods = new HashSet(10);
  Token mod = null;
}
:( mod = modifier { mods.add(mod.getText()); } )*
  ;

// inherited from grammar JavaRecognizer
typeSpec returns [Token t]{
  t = null;
}
:t=classTypeSpec
  | t=builtInTypeSpec
  ;

// inherited from grammar JavaRecognizer
classTypeSpec returns [Token id]{
  id = null;
}
:id=identifier (LBRACK RBRACK { id.setText(id.getText() + "[]"); } )*
  ;

// inherited from grammar JavaRecognizer
builtInTypeSpec returns [Token id]{
  id = null;
}
:id=builtInType (LBRACK RBRACK { id.setText(id.getText() + "[]"); } )*
  ;

// inherited from grammar JavaRecognizer
type returns [Token t]{
  t = null;
}
:t=identifier
  | t=builtInType
  ;

// inherited from grammar JavaRecognizer
builtInType returns [Token t]{ t = null; }
:tvoid:"void" { t = tvoid; }
  | tboolean:"boolean" { t = tboolean; }
  | tbyte:"byte" { t = tbyte; }
  | tchar:"char" { t = tchar; }
  | tshort:"short" { t = tshort; }
  | tint:"int" { t = tint; }
  | tfloat:"float" { t = tfloat; }
  | tlong:"long" { t = tlong; }
  | tdouble:"double" { t = tdouble; }
  ;

// inherited from grammar JavaRecognizer
identifier returns [Token t]{ t = null; }
:id:IDENT { t = id; }
    ( DOT id2:IDENT { t.setText(t.getText() + "." + id2.getText()); } )*
  ;

// inherited from grammar JavaRecognizer
identifierStar {
  String className = "";
  String packageName = "";
}
:id:IDENT        { className = id.getText(); }
    ( DOT id2:IDENT
     { packageName += "." + className; className = id2.getText(); } )*
    ( DOT STAR      { packageName += "." + className; className = null; } )?

    {
     // tell the symbol table about the import
     getSymtab().addImport(className, packageName);
    }
  ;

// inherited from grammar JavaRecognizer
modifier returns [Token t]{
  t = null;
}
:tprivate:"private" { t = tprivate; }
  | tpublic:"public" { t = tpublic; }
  | tprotected:"protected" { t = tprotected; }
  | tstatic:"static" { t = tstatic; }
  | ttransient:"transient" { t = ttransient; }
  | tfinal:"final" { t = tfinal; }
  | tabstract:"abstract" { t = tabstract; }
  | tnative:"native" { t = tnative; }
  | tthreadsafe:"threadsafe" { t = tthreadsafe; }
  | tsynchronized:"synchronized" { t = tsynchronized; }
    //  |   tconst:"const"  { t = tconst; }     // reserved word; leave out
  | tvolatile:"volatile" { t = tvolatile; }
  ;

// inherited from grammar JavaRecognizer
classDefinition {
  String name = null;
  Token superclass = null;
}
:"class" id:IDENT
    {
      if(getSymtab().getCurrentClass() != null) {
    name = getSymtab().getCurrentClass().getName() + "$" + id.getText();
      }
      else {
    name = id.getText();
      }
    }
    // it _might_ have a superclass...
    superclass = superClassClause
    // it might implement some interfaces...
    implementsClause
    // now parse the body of the class
    classBlock[name, false, false, superclass]
  ;

// inherited from grammar JavaRecognizer
superClassClause returns [Token id]{ id = null; }
:( "extends" id=identifier )?
  ;

// inherited from grammar JavaRecognizer
interfaceDefinition {
  String name = null;
}
:"interface" id:IDENT
    {
      if(getSymtab().getCurrentClass() != null) {
    name = getSymtab().getCurrentClass().getName() + "$" + id.getText();
      }
      else {
    name = id.getText();
      }
    }
    
    // it might extend some other interfaces
    interfaceExtends
    // now parse the body of the interface (looks like a class...)
    classBlock[name, true, false, null]
  ;

// inherited from grammar JavaRecognizer
classBlock[ String name, boolean isInterface, boolean isAnonymous, Token superclass ] :lc:LCURLY
    {
      getSymtab().startClass(name, getInvariants(), isInterface, isAnonymous, (superclass == null ? null : superclass.getText()));
      clearInvariants();
    }
    //this should just be methods and constructors,
    //but can't find a better place for it.
    ( prePostField | SEMI )*
    rc:RCURLY
    {
      clearInvariants();
      getSymtab().finishClass(new CodePoint(rc.getLine(), rc.getColumn()-1));
    }
  ;

// inherited from grammar JavaRecognizer
prePosts :( post:POST_CONDITION { addPostCondition(post); }
    | pre:PRE_CONDITION { addPreCondition(pre); }
    | ASSERT_CONDITION
    | INVARIANT_CONDITION
    )*
  ;

// inherited from grammar JavaRecognizer
prePostField :(JAVADOC_OPEN prePosts JAVADOC_CLOSE)* field
  ;

// inherited from grammar JavaRecognizer
interfaceExtends { Token id, id2; }
:(
      "extends"
      id=identifier ( COMMA id2=identifier )*
    )?
  ;

// inherited from grammar JavaRecognizer
implementsClause { Token id, id2; }
:(
      "implements" id=identifier { parseImplementedInterface(id); }
      ( COMMA id2=identifier { parseImplementedInterface(id2); } )*
    )?
  ;

// inherited from grammar JavaRecognizer
field {
  Set mods = null;
  Token retType = null;
  List params = null;
  boolean methodOrConstructor = false;
  CodePointPair startEnd = null;
  CodePointPair dummyStartEnd;
  Set thrownExceptions = null;
  Pair p = null;
}
:(
      // method, constructor, or variable declaration
      //need to do something special for abstract and native methods here
      mods=modifiers
      ( p=ctorHead
    {
      params = (List)p.getOne();
      thrownExceptions = (Set)p.getTwo();
      // needs to be before compoundStatement so that I can have it set for the addExit calls
      getSymtab().startMethod(null, getPreConditions(), getPostConditions(), params, null, mods);
      clearPreConditions();
      clearPostConditions();
      //print("just called startMethod for constructor");
    }
    startEnd=compoundStatement // constructor
    {
      getSymtab().finishMethod(startEnd, thrownExceptions);
      //print("Found finish: " + methodName);
    }
    

      | classDefinition       // inner class
    
      | interfaceDefinition   // inner interface

      | retType=typeSpec  // method or variable declaration(s)
    (   methodName:IDENT  // the name of the method
      
      // parse the formal parameter declarations.
      LPAREN params=parameterDeclarationList
      {
        // needs to be before compoundStatement so that I can have it set for the addExit calls
        getSymtab().startMethod(methodName.getText(), getPreConditions(), getPostConditions(), params, retType.getText(), mods);
        //print("just called startMethod: " + methodName.getText());
        clearPreConditions();
        clearPostConditions();
      }
      RPAREN

      returnTypeBrackersOnEndOfMethodHead

      // get the list of exceptions that this method is declared to throw
      (thrownExceptions=throwsClause)?
      ( startEnd=compoundStatement | semi:SEMI )
      {
        if(startEnd != null) {
          getSymtab().finishMethod(startEnd, thrownExceptions);
        }
        else {
          //abstract, native or interface method
          CodePoint close = new CodePoint(semi.getLine(), semi.getColumn()-1);
          getSymtab().finishMethod(new CodePointPair(close, close), null);
        }
        //print("Found finish: " + methodName);
      }
    |   variableDefinitions SEMI
    )
      )

      // "static { ... }" class initializer
    |   "static" dummyStartEnd=compoundStatement

      // "{ ... }" instance initializer
    |   dummyStartEnd=compoundStatement
    )
  ;

// inherited from grammar JavaRecognizer
variableDefinitions :variableDeclarator
    (   COMMA
      variableDeclarator
    )*
  ;

// inherited from grammar JavaRecognizer
variableDeclarator :IDENT declaratorBrackets varInitializer
  ;

// inherited from grammar JavaRecognizer
declaratorBrackets :(lb:LBRACK RBRACK)*
  ;

// inherited from grammar JavaRecognizer
varInitializer :( ASSIGN initializer )?
  ;

// inherited from grammar JavaRecognizer
arrayInitializer :LCURLY
    (   initializer
      (
    // CONFLICT: does a COMMA after an initializer start a new
    //           initializer or start the option ',' at end?
    //           ANTLR generates proper code by matching
    //           the comma as soon as possible.
    options {
      warnWhenFollowAmbig = false;
    }
      :
    COMMA initializer
      )*
      (COMMA)?
    )?
    RCURLY
  ;

// inherited from grammar JavaRecognizer
initializer :expression
  | arrayInitializer
  ;

// inherited from grammar JavaRecognizer
ctorHead returns [Pair p]{
  List params = null;
  Set thrownExceptions = null;
  p = null;
}
:(
      IDENT  // the name of the method
      
      // parse the formal parameter declarations.
      LPAREN params=parameterDeclarationList RPAREN
      
      // get the list of exceptions that this method is declared to throw
      (thrownExceptions=throwsClause)?
    )
    
    {
      p = new Pair(params, thrownExceptions);
    }
  ;

// inherited from grammar JavaRecognizer
throwsClause returns [Set exceptions]{
  exceptions = new LinkedHashSet();
  Token id, id2;
}
:"throws" id=identifier { exceptions.add(id.getText()); }
    ( COMMA id2=identifier {exceptions.add(id2.getText()); } )*
  ;

// inherited from grammar JavaRecognizer
returnTypeBrackersOnEndOfMethodHead :(LBRACK RBRACK)*
  ;

// inherited from grammar JavaRecognizer
parameterDeclarationList returns [List params]{
  params = new LinkedList();
  StringPair pd = null;
}
:( pd=parameterDeclaration { params.add(pd); } ( COMMA pd=parameterDeclaration { params.add(pd); } )* )?
  ;

// inherited from grammar JavaRecognizer
parameterDeclaration returns [StringPair sp]{
  StringBuffer brackets = null;
  Token type;
  sp = null;
}
:parameterModifier type=typeSpec name:IDENT
    brackets=parameterDeclaratorBrackets
    {
      String typeText = type.getText();
      if(brackets.length() > 0) {
        typeText += brackets.toString();
      }
      sp = new StringPair(typeText, name.getText());
    }
  ;

// inherited from grammar JavaRecognizer
parameterDeclaratorBrackets returns [StringBuffer text]{
  text = new StringBuffer();
}
:(lb:LBRACK rb:RBRACK { text.append("[]"); } )*
  ;

// inherited from grammar JavaRecognizer
parameterModifier :("final")?
  ;

// inherited from grammar JavaRecognizer
assertOrInvariantCondition { List assertTokens = new LinkedList(); }
:(JAVADOC_OPEN
    ( assertCondition:ASSERT_CONDITION { assertTokens.add(assertCondition); clearInvariants(); }
      | PRE_CONDITION
      | POST_CONDITION
      | iv:INVARIANT_CONDITION { addInvariant(iv); assertTokens = new LinkedList(); }
    )*
    jdc:JAVADOC_CLOSE 
    { addAsserts(assertTokens, jdc); }
      )
  ;

// inherited from grammar JavaRecognizer
compoundStatement returns [CodePointPair startEnd]{
  startEnd = null;
}
:(
      lc:LCURLY
      // include the (possibly-empty) list of statements
      (statement)*
      rc:RCURLY
    )
    {
      CodePoint start = new CodePoint(lc.getLine(), lc.getColumn()-1);
      CodePoint end = new CodePoint(rc.getLine(), rc.getColumn()-1);
      startEnd = new CodePointPair(start, end);
    }
  ;

// inherited from grammar JavaRecognizer
casesGroup :(   // CONFLICT: to which case group do the statements bind?
      //           ANTLR generates proper code: it groups the
      //           many "case"/"default" labels together then
      //           follows them with the statements
      options {
    warnWhenFollowAmbig = false;
      }
    :
      aCase
    )+
    caseSList
  ;

// inherited from grammar JavaRecognizer
aCase :("case" expression | "default") COLON
  ;

// inherited from grammar JavaRecognizer
caseSList :(statement)*
  ;

// inherited from grammar JavaRecognizer
forInit :(   (declaration)=> declaration
      // otherwise it could be an expression list...
    |   expressionList
    )?
  ;

// inherited from grammar JavaRecognizer
forCond :(expression)?
  ;

// inherited from grammar JavaRecognizer
forIter :(expressionList)?
  ;

// inherited from grammar JavaRecognizer
tryBlock {
  CodePointPair dummyStartEnd;
}
:"try" dummyStartEnd=compoundStatement
    (handler)*
    ( "finally" dummyStartEnd=compoundStatement )?
  ;

// inherited from grammar JavaRecognizer
handler {
  CodePointPair dummyStartEnd;
  StringPair dummyParams;
}
:"catch" LPAREN dummyParams=parameterDeclaration RPAREN dummyStartEnd=compoundStatement
  ;

// inherited from grammar JavaRecognizer
expression :assignmentExpression
  ;

// inherited from grammar JavaRecognizer
expressionList :expression (COMMA expression)*
  ;

// inherited from grammar JavaRecognizer
assignmentExpression :conditionalExpression
    (   (   ASSIGN
      |   PLUS_ASSIGN
      |   MINUS_ASSIGN
      |   STAR_ASSIGN
      |   DIV_ASSIGN
      |   MOD_ASSIGN
      |   SR_ASSIGN
      |   BSR_ASSIGN
      |   SL_ASSIGN
      |   BAND_ASSIGN
      |   BXOR_ASSIGN
      |   BOR_ASSIGN
      )
      assignmentExpression
    )?
  ;

// inherited from grammar JavaRecognizer
conditionalExpression :logicalOrExpression
    ( QUESTION conditionalExpression COLON conditionalExpression )?
  ;

// inherited from grammar JavaRecognizer
logicalOrExpression :logicalAndExpression (LOR logicalAndExpression)*
  ;

// inherited from grammar JavaRecognizer
logicalAndExpression :inclusiveOrExpression (LAND inclusiveOrExpression)*
  ;

// inherited from grammar JavaRecognizer
inclusiveOrExpression :exclusiveOrExpression (BOR exclusiveOrExpression)*
  ;

// inherited from grammar JavaRecognizer
exclusiveOrExpression :andExpression (BXOR andExpression)*
  ;

// inherited from grammar JavaRecognizer
andExpression :equalityExpression (BAND equalityExpression)*
  ;

// inherited from grammar JavaRecognizer
equalityExpression :relationalExpression ((NOT_EQUAL | EQUAL) relationalExpression)*
  ;

// inherited from grammar JavaRecognizer
relationalExpression {
  Token dummyType;
}
:shiftExpression
    (
      (
    (LT
    |   GT
    |   LE
    |   GE
    )
    shiftExpression
      )*
    |   "instanceof" dummyType=typeSpec
    )
  ;

// inherited from grammar JavaRecognizer
shiftExpression :additiveExpression ((SL | SR | BSR) additiveExpression)*
  ;

// inherited from grammar JavaRecognizer
additiveExpression :multiplicativeExpression ((PLUS | MINUS) multiplicativeExpression)*
  ;

// inherited from grammar JavaRecognizer
multiplicativeExpression :unaryExpression ((STAR | DIV | MOD ) unaryExpression)*
  ;

// inherited from grammar JavaRecognizer
unaryExpression :INC unaryExpression
  | DEC unaryExpression
  | MINUS unaryExpression
  | PLUS  unaryExpression
  | unaryExpressionNotPlusMinus
  ;

// inherited from grammar JavaRecognizer
unaryExpressionNotPlusMinus {
  Token dummyToken;
}
:BNOT unaryExpression
  | LNOT unaryExpression

  | (   // subrule allows option to shut off warnings
      options {
        // "(int" ambig with postfixExpr due to lack of sequence
        // info in linear approximate LL(k).  It's ok.  Shut up.
        generateAmbigWarnings=false;
      }
    :   // If typecast is built in type, must be numeric operand
      // Also, no reason to backtrack if type keyword like int, float...
      LPAREN dummyToken=builtInTypeSpec RPAREN
      unaryExpression

      // Have to backtrack to see if operator follows.  If no operator
      // follows, it's a typecast.  No semantic checking needed to parse.
      // if it _looks_ like a cast, it _is_ a cast; else it's a "(expr)"
    |   (LPAREN classTypeSpec RPAREN unaryExpressionNotPlusMinus)=>
      LPAREN dummyToken=classTypeSpec RPAREN
      unaryExpressionNotPlusMinus

    |   postfixExpression
    )
  ;

// inherited from grammar JavaRecognizer
postfixExpression { Token id; }
:primaryExpression // start with a primary

    (   // qualified id (id.id.id.id...) -- build the name
      DOT ( IDENT
      | "this"
      | "class"
      | newExpression
      | "super" LPAREN ( expressionList )? RPAREN
      )
      // the above line needs a semantic check to make sure "class"
      //   is the _last_ qualifier.

      // allow ClassName[].class
    |   ( LBRACK RBRACK )+
      DOT "class"

      // an array indexing operation
    |   LBRACK expression RBRACK

      // method invocation
      // The next line is not strictly proper; it allows x(3)(4) or
      //  x[2](4) which are not valid in Java.  If this grammar were used
      //  to validate a Java program a semantic check would be needed, or
      //   this rule would get really ugly...
    |   LPAREN
      argList
      RPAREN
    )*

    // possibly add on a post-increment or post-decrement.
    // allows INC/DEC on too much, but semantics can check
    (   INC
    |   DEC
    |   // nothing
    )

    // look for int.class and int[].class
  | id=builtInType 
    ( LBRACK RBRACK )*
    DOT "class"
  ;

// inherited from grammar JavaRecognizer
primaryExpression :IDENT
  | newExpression
  | constant
  | "super"
  | "true"
  | "false"
  | "this"
  | "null"
  | LPAREN assignmentExpression RPAREN
  ;

// inherited from grammar JavaRecognizer
newExpression { Token t; }
:"new" t=type
    (   LPAREN argList RPAREN ( classBlock[null, false, true, t] )?

      //[jpschewe:20000128.0740CST] FIX need to use t to figure out what
      //interfaces we need to check conditions on.
        
      //java 1.1
      // Note: This will allow bad constructs like
      //    new int[4][][3] {exp,exp}.
      //    There needs to be a semantic check here...
      // to make sure:
      //   a) [ expr ] and [ ] are not mixed
      //   b) [ expr ] and an init are not used together

    |   newArrayDeclarator (arrayInitializer)?
    )
  ;

// inherited from grammar JavaRecognizer
argList :(   expressionList
    |   /*nothing*/
    )
  ;

// inherited from grammar JavaRecognizer
newArrayDeclarator :(
      // CONFLICT:
      // newExpression is a primaryExpression which can be
      // followed by an array index reference.  This is ok,
      // as the generated code will stay in this loop as
      // long as it sees an LBRACK (proper behavior)
      options {
    warnWhenFollowAmbig = false;
      }
    :
      LBRACK
      (expression)?
      RBRACK
    )+
  ;

// inherited from grammar JavaRecognizer
constant :NUM_INT
  | CHAR_LITERAL
  | STRING_LITERAL
  | NUM_FLOAT
  ;


